@@ -45,6 +45,7 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 
+#include <linux/version.h>
 #include <linux/connector.h>
 #include <linux/netlink.h>
 #include <linux/cn_proc.h>
@@ -83,29 +84,30 @@ typedef struct {
 	size_t len;		/* Length */
 } kernel_task_info;
 
-enum {
+typedef enum {
 	STAT_FORK = 0,
 	STAT_EXEC,
 	STAT_EXIT,
 	STAT_CORE,
 	STAT_COMM,
 	STAT_LAST
-};
+} event_t;
 
 typedef struct proc_stats {
-	char *name;
-	uint64_t count[STAT_LAST];
-	uint64_t total;
-	struct proc_stats *next;
+	char *name;		/* Process name */
+	uint64_t count[STAT_LAST]; /* Tally count */
+	uint64_t total;		/* Tally count total of all counts */
+	struct proc_stats *next;/* Next one in list */
 } proc_stats_t;
 
 typedef struct {
-	const char *event;
-	const char *label;
-	const int flag;
-	const int stat;
+	const char *event;	/* Event name */
+	const char *label;	/* Human readable label */
+	const int flag;		/* option flag */
+	const event_t stat;	/* stat enum */
 } ev_map_t;
 
+/* Mapping of event names to option flags and event_t types */
 static const ev_map_t ev_map[] = {
 	{ "fork", "Fork", OPT_EV_FORK, STAT_FORK },
 	{ "exec", "Exec", OPT_EV_EXEC, STAT_EXEC },
@@ -180,7 +182,7 @@ static bool sane_proc_pid_info(void)
 
 /*
  *  pid_a_kernel_thread
- *
+ *	is a process a kernel thread?
  */
 static bool pid_a_kernel_thread(const char *task, const pid_t id)
 {
@@ -255,6 +257,10 @@ static int tty_height(void)
 	return 25;	/* else standard tty 80x25 */
 }
 
+/*
+ *  print_heading()
+ *	print heading to output
+ */
 static void print_heading(void)
 {
 	if (opt_flags & OPT_QUIET)
@@ -317,7 +323,7 @@ static inline int proc_name_hash(const char *str)
 	return h % MAX_PIDS;
 }
 
-static void proc_stats_account(pid_t pid, int event)
+static void proc_stats_account(const pid_t pid, const event_t event)
 {
 	int h;
 	char *name;
@@ -357,6 +363,10 @@ static void proc_stats_account(pid_t pid, int event)
 	proc_stats[h] = stats;
 }
 
+/*
+ *  stats_cmp()
+ *	compare stats total, used for sorting list
+ */
 int stats_cmp(const void *v1, const void *v2)
 {
 	proc_stats_t **s1 = (proc_stats_t **)v1;
@@ -365,6 +375,10 @@ int stats_cmp(const void *v1, const void *v2)
 	return (*s2)->total - (*s1)->total;
 }
 
+/*
+ *  proc_stats_report()
+ *	report event statistics
+ */
 void proc_stats_report(void)
 {
 	int i;
@@ -410,6 +424,10 @@ void proc_stats_report(void)
 	free(sorted);
 }
 
+/*
+ *  proc_stats_free()
+ *	free stats list
+ */
 void proc_stats_free(void)
 {
 	int i;
@@ -632,7 +650,7 @@ static proc_info_t *proc_info_add(const pid_t pid, struct timeval *tv)
  *  proc_thread_info_add()
  *	Add a processes' thread into proc cache
  */
-static void proc_thread_info_add(pid_t pid)
+static void proc_thread_info_add(const pid_t pid)
 {
 	DIR *dir;
 	struct dirent *dirent;
@@ -645,8 +663,11 @@ static void proc_thread_info_add(pid_t pid)
 
 	while ((dirent = readdir(dir))) {
 		if (isdigit(dirent->d_name[0])) {
-			pid_t tpid = atoi(dirent->d_name);
-			if (tpid != pid)
+			pid_t tpid;
+
+			errno = 0;
+			tpid = (pid_t)strtol(dirent->d_name, NULL, 10);
+			if ((!errno) && (tpid != pid))
 				(void)proc_info_add(tpid, NULL);
 		}
 	}
@@ -668,9 +689,13 @@ static int proc_info_load(void)
 
 	while ((dirent = readdir(dir))) {
 		if (isdigit(dirent->d_name[0])) {
-			pid_t pid = atoi(dirent->d_name);
+			pid_t pid;
+
+			errno = 0;
+			pid = (pid_t)strtol(dirent->d_name, NULL, 10);
 			(void)proc_info_add(pid, NULL);
-			proc_thread_info_add(pid);
+			if (!errno)
+				proc_thread_info_add(pid);
 		}
 	}
 
@@ -768,10 +793,22 @@ static int monitor(const int sock)
 			return 0;
 		}
 		if (len == -1) {
-			if (errno == EINTR) {
+			switch (errno) {
+			case EINTR:
 				return 0;
-			} else {
-				fprintf(stderr,"recv: %s\n", strerror(errno));
+			case ENOBUFS: {
+				time_t now;
+				struct tm tm;
+
+				(void)localtime_r(&now, &tm);
+				printf("%2.2d:%2.2d:%2.2d recv ----- "
+					"nobufs %8.8s (%s)\n",
+					tm.tm_hour, tm.tm_min, tm.tm_sec, "",
+					strerror(errno));
+				break;
+			}
+			default:
+				fprintf(stderr,"recv: %d %s\n", errno, strerror(errno));
 				return -1;
 			}
 		}
@@ -803,11 +840,11 @@ static int monitor(const int sock)
 
 			(void)time(&now);
 			(void)localtime_r(&now, &tm);
-
 			snprintf(when, sizeof(when), "%2.2d:%2.2d:%2.2d",
 				tm.tm_hour, tm.tm_min, tm.tm_sec);
 
 			switch (proc_ev->what) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 			case PROC_EVENT_FORK:
 				proc_stats_account(proc_ev->event_data.fork.parent_pid, STAT_FORK);
 				gettimeofday(&tv, NULL);
@@ -874,6 +911,8 @@ static int monitor(const int sock)
 				}
 				proc_info_free(proc_ev->event_data.exit.process_pid);
 				break;
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
 			case PROC_EVENT_COREDUMP:
 				proc_stats_account(proc_ev->event_data.coredump.process_pid, STAT_CORE);
 				if (!(opt_flags & OPT_QUIET) && (opt_flags & OPT_EV_CORE)) {
@@ -888,11 +927,13 @@ static int monitor(const int sock)
 						info1->kernel_thread ? "]" : "");
 				}
 				break;
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,1,0)
 			case PROC_EVENT_COMM:
 				proc_stats_account(proc_ev->event_data.comm.process_pid, STAT_COMM);
 				if (!(opt_flags & OPT_QUIET) && (opt_flags & OPT_EV_COMM)) {
 					info1 = proc_info_get(proc_ev->event_data.comm.process_pid);
-					comm = proc_comm(proc_ev->event_data.coredump.process_pid);
+					comm = proc_comm(proc_ev->event_data.comm.process_pid);
 					if (comm == NULL)
 						break;
 					row_increment();
@@ -908,6 +949,7 @@ static int monitor(const int sock)
 					free(comm);
 				}
 				break;
+#endif
 			default:
 				break;
 			}
